#pragma once

#include <cstdio>
#include "execution_defs.h"
#include "execution_manager.h"
#include "executor_abstract.h"
#include "index/ix.h"
#include "system/sm.h"

class InsertExecutor : public AbstractExecutor {
private:
    TabMeta tab_;
    std::vector<Value> values_;
    RmFileHandle *fh_;
    std::string tab_name_;
    Rid rid_{};
    SmManager *sm_manager_;

public:
    InsertExecutor(SmManager *sm_manager, const std::string &tab_name, std::vector<Value> values, Context *context) {
        sm_manager_ = sm_manager;
        tab_ = sm_manager_->db_.get_table(tab_name);
        values_ = std::move(values);
        tab_name_ = tab_name;
        fh_ = sm_manager_->fhs_.at(tab_name).get();
        context_ = context;
    }

    std::unique_ptr<RmRecord> Next() override {
        // 构建记录
        RmRecord rec(fh_->get_file_hdr().record_size);
        for (size_t i = 0; i < values_.size(); i++) {
            auto &col = tab_.cols[i];
            auto &val = values_[i];
            if (col.type != val.type) {
                throw IncompatibleTypeError(coltype2str(col.type), coltype2str(val.type));
            }
            val.init_raw(col.len);
            memcpy(rec.data + col.offset, val.raw->data, col.len);
        }

        // 获取插入位置的间隙范围
        Rid start_rid, end_rid;
        std::tie(start_rid, end_rid) = get_insert_gap_range(rec);

        // 申请间隙锁（GAP LOCK）
        if (context_ != nullptr && start_rid != Rid::Invalid() && end_rid != Rid::Invalid()) {
            context_->lock_mgr_->lock_exclusive_on_gap(context_->txn_, start_rid, end_rid, fh_->GetFd());
        }

        // 插入记录到文件并获取新的rid
        rid_ = fh_->insert_record(rec.data, context_);

        // 申请行级排他锁（X锁）
        context_->lock_mgr_->lock_exclusive_on_record(context_->txn_, rid_, fh_->GetFd());

        // 记录事务日志
//        Transaction *txn = context_->txn_;
//        auto *insert_log = new InsertLogRecord(txn->get_transaction_id(), rec, rid_, tab_name_);
//        insert_log->prev_lsn_ = txn->get_prev_lsn();
//        context_->log_mgr_->add_insert_log_record(context_->txn_->get_transaction_id(), rec, rid_, disk_manager_->get_file_name(fd_));
//
//        txn->set_prev_lsn(context_->log_mgr_->add_log_to_buffer(insert_log));
//
//        auto *write_rcd = new TableWriteRecord(WType::INSERT_TUPLE, tab_name_, rid_);
//        context_->txn_->append_table_write_record(write_rcd);

        // 插入索引条目
        for (auto &index: tab_.indexes) {
            auto ih = sm_manager_->ihs_.at(
                    sm_manager_->get_ix_manager()->get_index_name(tab_name_, index.cols)).get();

            std::vector<char> key(index.col_tot_len);
            int offset = 0;
            for (size_t i = 0; i < index.col_num; ++i) {
                memcpy(key.data() + offset, rec.data + index.cols[i].offset, index.cols[i].len);
                offset += index.cols[i].len;
            }
            ih->insert_entry(key.data(), rid_, context_->txn_);
            auto *index_rcd = new IndexWriteRecord(WType::INSERT_TUPLE, tab_name_, rid_, key.data(), index.col_tot_len);
            context_->txn_->append_index_write_record(index_rcd);
        }
        return nullptr;
    }

    Rid &rid() override { return rid_; }

    std::pair<Rid, Rid> get_insert_gap_range(const RmRecord &rec) {
        // 检查表是否存在索引
        if (tab_.indexes.empty()) {
            return std::make_pair(Rid::Invalid(), Rid::Invalid());
        }

        // 假设使用表的第一个索引（通常是主键或唯一索引）
        auto &index = tab_.indexes[0];
        auto ih = sm_manager_->ihs_.at(
                sm_manager_->get_ix_manager()->get_index_name(tab_name_, index.cols)).get();

        // 构建插入记录的键
        std::vector<char> key(index.col_tot_len);
        int offset = 0;
        for (size_t i = 0; i < index.col_num; ++i) {
            memcpy(key.data() + offset, rec.data + index.cols[i].offset, index.cols[i].len);
            offset += index.cols[i].len;
        }

        // 查找插入位置前后的 RID
        Iid lower_iid = ih->lower_bound(key.data());
        Iid upper_iid = ih->upper_bound(key.data());

        Rid start_rid = {lower_iid.page_no, lower_iid.slot_no};
        Rid end_rid = {upper_iid.page_no, upper_iid.slot_no};

        // 如果没有找到合适的前后间隙 RID，可以设置为无效的 RID
        if (start_rid.page_no == -1 && start_rid.slot_no == -1) {
            start_rid = Rid::Invalid();
        }
        if (end_rid.page_no == -1 && end_rid.slot_no == -1) {
            end_rid = Rid::Invalid();
        }

        return std::make_pair(start_rid, end_rid);
    }
};